@@ -1,17 +1,19 @@
+from dataclasses import dataclass
 from enum import Enum
 from string import Template
-from typing import Callable, Dict, List, NamedTuple, Union
+from typing import Callable, Dict, List, NamedTuple, Set, Union
 
-from pyqir.generator.types import Double, Integer, Qubit, Result, Void  # type: ignore
+from pyqir.generator.types import Double, Int, Qubit, Result, Void  # type: ignore
+from pytket import OpType  # type: ignore
 
-
-PyQirInputTypes = Union[Double, Qubit, Result]
-PyQirOutputTypes = Union[Integer, Result, Void]
+PyQirParameterType = Union[Double, Int, Qubit, Result]
+PyQirReturnType = Union[Int, Result, Void]
 
 
 class OpNat(Enum):
     QIS = "qis"
     CIS = "cis"
+    HYBRID = "hybrid"
 
 
 class OpName(Enum):
@@ -45,60 +47,17 @@ class OpSpec(Enum):
     CTLADJ = "ctladj"
 
 
-QirGate = NamedTuple(
-    "QirGate",
-    [
-        (
-            "opnat",
-            OpNat,
-        ),
-        (
-            "opname",
-            OpName,
-        ),
-        (
-            "opspec",
-            OpSpec,
-        ),
-    ],
-)
+@dataclass(frozen=True)
+class QirGate:
+    opnat: OpNat
+    opname: Union[OpName, Enum]
+    opspec: OpSpec
 
 
-CustomQirGate = NamedTuple(
-    "CustomQirGate",
-    [
-        (
-            "opnat",
-            OpNat,
-        ),
-        (
-            "opname",
-            OpName,
-        ),
-        (
-            "opspec",
-            OpSpec,
-        ),
-        (
-            "function_signature",
-            List[PyQirInputTypes],
-        ),
-        (
-            "return_type",
-            PyQirOutputTypes,
-        ),
-    ],
-)
-
-
-GateSet = NamedTuple(
-    "GateSet",
-    [
-        ("name", str),
-        ("tk_to_gateset", Callable),
-        ("gateset_to_tk", Callable),
-    ],
-)
+@dataclass(frozen=True)
+class CustomQirGate(QirGate):
+    function_signature: List[PyQirParameterType]
+    return_type: PyQirReturnType
 
 
 CustomGateSet = NamedTuple(
@@ -106,6 +65,7 @@ CustomGateSet = NamedTuple(
     [
         ("name", str),
         ("template", Template),
+        ("base_gateset", Set[OpType]),
         ("gateset", Dict[str, CustomQirGate]),
         ("tk_to_gateset", Callable),
         ("gateset_to_tk", Callable),
